# -*- coding: utf-8 -*-
from zope.component import testing

import doctest
import re
import six
import unittest


class TestWrapperUpdate(unittest.TestCase):

    def test_wrapper_update(self):
        from plone.indexer import indexer
        from zope.interface import Interface

        @indexer(Interface)
        def my_func(obj):
            """My custom docstring."""

        self.assertEqual(my_func.__doc__, 'My custom docstring.')
        self.assertEqual(my_func.__module__, 'plone.indexer.tests')
        self.assertEqual(my_func.__name__, 'my_func')


class Py23DocChecker(doctest.OutputChecker):
    def check_output(self, want, got, optionflags):
        if six.PY2:
            got = re.sub("u'(.*?)'", "'\\1'", got)
        return doctest.OutputChecker.check_output(self, want, got, optionflags)


def test_suite():
    return unittest.TestSuite([
        doctest.DocFileSuite(
            'README.rst',
            package='plone.indexer',
            setUp=testing.setUp,
            tearDown=testing.tearDown,
            checker=Py23DocChecker(),
            ),
        unittest.makeSuite(TestWrapperUpdate),
    ])


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



# -*- coding: utf-8 -*-
from zope.interface import Interface


# NOTE: CMF 2.2 provides standard interfaces for indexing. We provide
# compatibility aliases here for CMF 2.1

try:
    from Products.CMFCore.interfaces import IIndexableObjectWrapper
except ImportError:
    class IIndexableObjectWrapper(Interface):
        pass
try:
    from Products.CMFCore.interfaces import IIndexableObject
except ImportError:
    class IIndexableObject(Interface):
        """An object being indexed in the catalog. The indexable object
        wrapper will be looked up as a multi-adapter of (object, catalog)
        to this interface if the object being indexed in the catalog does
        not already provide this interface.
        """


class IIndexer(Interface):
    """A component that can provide the value for an catalog index.

    Register a named adapter from an indexable object type (e.g. a content
    object) and the catalog to this interface.

    The name of the adapter should be the same as the name of the indexable
    attribute in the catalog.

    See also decorator.py, for a simpler indexer based on a function
    decorator.
    """

    def __call__(self):
        """Return the value to index.
        """


class IDelegatingIndexableObjectWrapper(IIndexableObjectWrapper):
    """An adapter of a (object, catalog) where object is to be indexed in
    the catalog and catalog is the portal_catalog or similar ZCatalog instance.

    The catalog will call getattr() on the wrapper for each attribute to be
    indexed. The wrapper may either implement these directly (as methods
    taking no parameters) or implement __getattr__() appropriately.
    """

    def _getWrappedObject():
        """Return the object that was wrapped.

        This has a leading underscore to reduce the risk of clashing with
        an index or metadata column.
        """



# -*- coding: utf-8 -*-
"""This module defines a decorator that
"""

from plone.indexer.delegate import DelegatingIndexerFactory
from Products.ZCatalog.interfaces import IZCatalog
from zope.component import adapter


class indexer(adapter):
    """The @indexer decorator can be used like this:

        >>> from plone.indexer.decorator import indexer
        >>> @indexer(IMyType)
        ... def some_attribute(object):
        ...     return 'some indexable value'

    Note that the @indexer decorator is a superset of the @adapter decorator
    from zope.component.

    To register an indexer for a special type of catalog, use:

        >>> from plone.indexer.decorator import indexer
        >>> @indexer(IMyType, IMyCatalog)
        ... def some_attribute(object):
        ...     return 'some indexable value'

    The default is to register the indexer for all IZCatalog catalogs.

    Once you've created an indexer, you can register the adapter in ZCML:

        <adapter factory=".myindexers.some_attribute" name="some_attribute" />

    At this point, the indexable object wrapper will ensure that when
    some_attribute is indexed on an object providing IMyType
    """

    def __init__(self, *interfaces):
        if len(interfaces) == 1:
            interfaces += (IZCatalog, )
        elif len(interfaces) > 2:
            raise ValueError(
                u'The @indexer decorator takes at most two interfaces as '
                u'arguments.',
            )
        adapter.__init__(self, *interfaces)

    def __call__(self, callable):
        factory = DelegatingIndexerFactory(callable)
        return adapter.__call__(self, factory)



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

import os


version = '1.0.7'
description = 'Hooks to facilitate managing custom index values in Zope 2/CMF applications'  # noqa
long_description = ('\n\n'.join([
    open('README.rst').read(),
    open('CHANGES.rst').read(),
    open(os.path.join("plone", "indexer", "README.rst")).read(),

]))


setup(
    name='plone.indexer',
    version=version,
    description=description,
    long_description=long_description,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone :: 5.1",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: Core",
        "Framework :: Zope2",
        "Framework :: Zope :: 4",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    keywords='plone cmf zope catalog index',
    author='Plone Foundation',
    author_email='plone-developers@lists.sourceforge.net',
    url='https://pypi.org/project/plone.indexer',
    license='BSD',
    packages=find_packages(),
    namespace_packages=['plone'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'zope.interface',
        'zope.component',
        'Products.CMFCore',
        'Products.ZCatalog',
    ],
)
